#!/usr/bin/python
# -*- coding:utf-8 -*-

import os, string, re, sys, tempfile

# retrieve index page and resolve urls

def readURL( url, encoding = "sjis", filename_output = None ):
    filename_org = tempfile.mktemp()
    if filename_output == None:
        filename_output = tempfile.mktemp( ".html", "loto" )
        pass
    os.system( "curl \"%s\" -o \"%s\" -s" % ( url, filename_org ) )
    os.system( "iconv -f \"%s\" -t utf-8 \"%s\" > \"%s\"" % ( encoding, filename_org, filename_output ))
    os.unlink( filename_org )
    return filename_output

baseURL = "http://www.takarakuji.mizuhobank.co.jp/"

class LotoResult:
    def __init__( self ):
        self.title = ""
        self.date = ""
        self.totalsales = 0
        self.carryover = 0
        self.numbers = 0
        self.awards = []
        self.bonus = None
        self.lots = []
        pass

    def __repr__( self ):
        output = "T:%d;D:%s;N:%d"  % ( self.title, self.date, self.numbers[ 0 ] )
        
        for i in range( 1, len( self.numbers ) ):
            output += ",%d" % ( self.numbers[ i ] )
            pass
        if self.bonus:
            output += ";B:%d" % ( self.bonus )
            pass
        output += ";A:%d/%d" % ( self.awards[ 0 ], self.lots[ 0 ] )
        for i in range( 1, min( len( self.awards ), len( self.lots ) ) ):
            output += ",%d/%d" % ( self.awards[ i ], self.lots[ i ] )
            pass
        output += ";C:%d" % ( self.carryover )
        output += ";S:%d" % ( self.totalsales )
        return output
    pass

class LotoInformation:
    def __init__( self, name, url ):
        self.name = name
        self.url = url
        self.results = []
        self.max_display = 3
        pass

    def __repr__( self ):
        output = "-----%s<%s>\n" % ( self.name, self.url )
        for i in range( min( len( self.results ), self.max_display ) ):
            output += repr( self.results[ i ] ) + "\n"
            pass
        output += "//------"
        return output

    def __read_money( self, line ):
        m = re.search( u'([\\d,]+)円', line )
        if m:
            nstr = re.sub( u'[^\\d]', u'', m.group( 1 ) )
            return string.atoi( nstr.encode( 'utf-8' ) )
        else:
            return None
        pass
    
    def load( self ):
        filename = readURL( self.url )
        lines = [ unicode( line.strip(), 'utf-8' ) for line in open( filename ).readlines() ]
        os.unlink( filename )
        i = 0
        re_b = re.compile( u'<b>(.*?)</b>', re.I )
        re_yen = re.compile( u'([\\d,]+)円' )
        re_lot = re.compile( u'([\\d,]+)口' )
        re_title = re.compile( u'<b>第(\\d+)回</b>', re.I )
        while i < len( lines ):
            mtitle = re_title.search( lines[ i ] )
            if mtitle:
                result = LotoResult()
                md = re_b.search( lines[ i + 1 ] )
                result.title = string.atoi( mtitle.group( 1 ) )
                result.date = re.sub( u'[^A-Z\\d]', u'.', md.group( 1 ) ).encode( 'utf-8' )
                i += 2
                lots = []
                awards = []
                numbers = []
                while i < len( lines ):
                    if lines[ i ].find( u'販売実績額' ) >= 0:
                        i += 1
                        while i < len( lines ):
                            money = self.__read_money( lines[ i ] )
                            i += 1
                            if money:
                                result.totalsales = money
                                break
                            pass
                        j = i
                        while j < len( lines ):
                            if re_title.search( lines[ j ] ): break
                            money = self.__read_money( lines[ j ] )
                            j += 1
                            if money:
                                result.carryover = money
                                break
                            pass
                        break
                    mb = re_b.search( lines[ i ] )
                    if mb:
                        my = re_yen.search( mb.group( 1 ) )
                        ml = re_lot.search( mb.group( 1 ) )
                        if my: awards.append( re.sub( u'[^\\d]', u'', my.group( 1 ) ).encode( 'utf-8' ) )
                        elif ml: lots.append( re.sub( u'[^\\d]', u'', ml.group( 1 ) ).encode( 'utf-8' ) )
                        else:
                            numbers.append( re.sub( u'[^\\d]', u'', mb.group( 1 ).encode( 'utf-8' ) ) )
                            pass
                        pass
                    i += 1
                    pass
                result.numbers = [ string.atoi( n ) for n in numbers ]
                result.awards = [ string.atoi( n ) for n in awards ]
                result.lots = [ string.atoi( n ) for n in lots ]

                if self.name == "LOTO6":
                    result.bonus = result.numbers[ 3 ]
                    result.numbers = result.numbers[ 0: 3 ] + result.numbers[ 4: ]
                elif self.name == "MiniLOTO":
                    result.bonus = result.numbers[ 5 ]
                    result.numbers = result.numbers[ 0: 5 ]
                    pass
                self.results.append( result )
                pass
            i += 1
            pass
        self.results.sort( lambda x, y: - ( x.title > y.title ) )
        pass
    pass

if __name__ == "__main__":
    filename_base = readURL( baseURL + "contents.html" )
    lotoinfo = []

    for line in open( filename_base ).readlines():
        if line.find( "最新回号" ) < 0: continue
        line = unicode( line.strip(), 'utf-8' )
        mel = re.search( u'<a (.*?)>(.*?)</a>', line, re.I )
        while mel != None:
            ml = re.search( u'href=\"([^\"]+)', mel.group( 1 ) )
            mi = re.search( u'<img\\W[^>]+', mel.group( 2 ), re.I )
            if mi != None and ml != None: 
                ma = re.search( u'alt=\"([^\"]+)', mi.group( 0 ) )
                if ma != None:
                    name = ma.group( 1 )
                    loto = None
                    url = ml.group( 1 ).encode( 'utf-8' )
                    if name == u'ロト６':
                        loto = LotoInformation( "LOTO6", baseURL + url )
                    elif name == u'ミニロト':
                        loto = LotoInformation( "MiniLOTO", baseURL + url )
                    elif name == u'ナンバーズ３':
                        loto = LotoInformation( "NUMBERS3", baseURL + url )
                    elif name == u'ナンバーズ４':
                        loto = LotoInformation( "NUMBERS4", baseURL + url )
                        pass
                    if loto: lotoinfo.append( loto )
                    pass
                pass
            line = line[ mel.end(): ]
            mel = re.search( u'<a (.*?)>(.*?)</a>', line, re.I )
            pass
        pass
    os.unlink( filename_base )

    for loto in lotoinfo:
        loto.load()
        print loto
        pass
    pass
